<?php
/**
 * Image Utility
 * @package     Me
 * @copyright   Copyright (C) 2011 MicroEngine Inc.
 */
class Me_ImageUtil
{

    /**
     * 画像作成
     * POSTしたファイルを、指定した名前で、指定した場所に、指定サイズにリサイズして保存する。
     * @param string $source_file
     * @param string $save_file_path
     * @param string $max_width
     * @param string $max_height
     * @param boolean $resize
     * @return array
     */
    static public function create_image($source_file, $save_file_path, $max_width, $max_height, $resize = true)
    {
        // アップロードファイルかどうかの確認
        if (!is_uploaded_file($source_file)) {
            throw new Me_Exception(604);
        }
        // maxサイズの確認
        if (!is_numeric($max_width)) {
            $max_width = '800';
        }
        if (!is_numeric($max_height)) {
            $max_height = '600';
        }

        // 画像の情報を取得
        $source_info = getimagesize($source_file);
        if ($source_info === false) {
            throw new Me_Exception(605);
        }
        list($source_width, $source_height, $source_type) = $source_info;

        // image resource を作成
        switch ($source_type) {
            case '1':
                $source_image = imagecreatefromgif($source_file);
                break;
            case '2':
                $source_image = imagecreatefromjpeg($source_file);
                break;
            case '3':
                $source_image = imagecreatefrompng($source_file);
                break;
            default:
                $resize = false;
                break;
        }

        // 作成する画像ピクセルサイズを計算
        $new_width = $source_width;
        $new_height = $source_height;

        // リサイズ比率を取得
        $ratio_width = Me_ImageUtil::get_resize_ratio($source_width, $max_width);
        $ratio_height = Me_ImageUtil::get_resize_ratio($source_height, $max_height);

        // 幅か高さのどちらかが最大サイズを超えていればリサイズする
        if ($ratio_width > 1 || $ratio_height > 1) {
            // より比率の大きいほうに合わせる
            if ($ratio_width > $ratio_height) {
                $new_width = $max_width;
                $new_height = round($source_height / $ratio_width);
            } else {
                $new_width = round($source_width / $ratio_height);
                $new_height = $max_height;
            }
        }

        // 新規画像を作成
        if ($source_type === '1') {
            $new_image = imagecreate($new_width, $new_height);
        } else {
            $new_image = imagecreatetruecolor($new_width, $new_height);
        }

        // リサンプル。イメージを縮小する。
        $result = imagecopyresampled(
                $new_image,
                $source_image,
                0,
                0,
                0,
                0,
                $new_width,
                $new_height,
                $source_width,
                $source_height
        );

        // image resourceを書き出す
        switch ($source_type) {
            case '1':
                imagegif($new_image, $save_file_path);
                break;
            case '2':
                imagejpeg($new_image, $save_file_path);
                break;
            case '3':
                imagepng($new_image, $save_file_path);
                break;
            default:
                break;
        }

        // image resourceを破棄する
        imagedestroy($new_image);
        imagedestroy($source_image);

        // 新画像サイズ取得
        return array($new_width, $new_height);
    }

    /**
     * リサイズ比率を返す
     * @param integer $original
     * @param integer $max
     * @return integer $ratio
     */
    static public function get_resize_ratio($original, $max)
    {
        $ratio = 0;
        // 0以上の数値でなければ、0を返す。
        if (!is_numeric($original) || $original === 0 || !is_numeric($max) || $max === 0) {
            return $ratio;
        }
        $ratio = $original / $max;
        return $ratio;
    }
}